@@ -3,7 +3,6 @@
 mod buffer;
 mod bytes;
 mod fragment;
-mod list;
 mod long;
 #[cfg(feature = "yyjson")]
 pub mod yyjson;
@@ -11,5 +10,4 @@ pub mod yyjson;
 pub use buffer::*;
 pub use bytes::*;
 pub use fragment::{orjson_fragmenttype_new, Fragment};
-pub use list::PyListIter;
 pub use long::{pylong_is_unsigned, pylong_is_zero};
