@@ -1,10 +1,10 @@
 use crate::opt::*;
 
-use crate::serialize::error::*;
-use crate::serialize::per_type::datetimelike::{
-    DateTimeBuffer, DateTimeError, DateTimeLike, Offset,
+use crate::serialize::error::SerializeError;
+use crate::serialize::per_type::{
+    DateTimeBuffer, DateTimeError, DateTimeLike, DefaultSerializer, Offset, ZeroListSerializer,
 };
-use crate::serialize::per_type::*;
+use crate::serialize::serializer::PyObjectSerializer;
 use crate::typeref::{load_numpy_types, ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR, NUMPY_TYPES};
 use chrono::{Datelike, NaiveDate, NaiveDateTime, Timelike};
 use pyo3_ffi::*;
@@ -12,35 +12,19 @@ use serde::ser::{self, Serialize, SerializeSeq, Serializer};
 use std::convert::TryInto;
 use std::fmt;
 use std::os::raw::{c_char, c_int, c_void};
-use std::ptr::NonNull;
 
-pub struct NumpySerializer {
-    ptr: *mut pyo3_ffi::PyObject,
-    opts: Opt,
-    default_calls: u8,
-    recursion: u8,
-    default: Option<NonNull<pyo3_ffi::PyObject>>,
+#[repr(transparent)]
+pub struct NumpySerializer<'a> {
+    previous: &'a PyObjectSerializer,
 }
 
-impl NumpySerializer {
-    pub fn new(
-        ptr: *mut pyo3_ffi::PyObject,
-        opts: Opt,
-        default_calls: u8,
-        recursion: u8,
-        default: Option<NonNull<pyo3_ffi::PyObject>>,
-    ) -> Self {
-        NumpySerializer {
-            ptr: ptr,
-            opts: opts,
-            default_calls: default_calls,
-            recursion: recursion,
-            default: default,
-        }
+impl<'a> NumpySerializer<'a> {
+    pub fn new(previous: &'a PyObjectSerializer) -> Self {
+        Self { previous: previous }
     }
 }
 
-impl Serialize for NumpySerializer {
+impl<'a> Serialize for NumpySerializer<'a> {
     #[cold]
     #[inline(never)]
     #[cfg_attr(feature = "optimize", optimize(size))]
@@ -48,20 +32,13 @@ impl Serialize for NumpySerializer {
     where
         S: Serializer,
     {
-        match NumpyArray::new(self.ptr, self.opts) {
+        match NumpyArray::new(self.previous.ptr, self.previous.state.opts()) {
             Ok(val) => val.serialize(serializer),
             Err(PyArrayError::Malformed) => err!(SerializeError::NumpyMalformed),
             Err(PyArrayError::NotContiguous) | Err(PyArrayError::UnsupportedDataType)
-                if self.default.is_some() =>
+                if self.previous.default.is_some() =>
             {
-                DefaultSerializer::new(
-                    self.ptr,
-                    self.opts,
-                    self.default_calls,
-                    self.recursion,
-                    self.default,
-                )
-                .serialize(serializer)
+                DefaultSerializer::new(self.previous).serialize(serializer)
             }
             Err(PyArrayError::NotContiguous) => {
                 err!(SerializeError::NumpyNotCContiguous)
@@ -79,7 +56,7 @@ macro_rules! slice {
     };
 }
 
-#[cfg_attr(feature = "optimize", optimize(size))]
+#[cold]
 pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
     let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
     if numpy_types.is_none() {
@@ -101,7 +78,7 @@ pub fn is_numpy_scalar(ob_type: *mut PyTypeObject) -> bool {
     }
 }
 
-#[cfg_attr(feature = "optimize", optimize(size))]
+#[cold]
 pub fn is_numpy_array(ob_type: *mut PyTypeObject) -> bool {
     let numpy_types = unsafe { NUMPY_TYPES.get_or_init(load_numpy_types) };
     if numpy_types.is_none() {
@@ -319,7 +296,7 @@ impl Serialize for NumpyArray {
         S: Serializer,
     {
         if unlikely!(!(self.depth >= self.dimensions() || self.shape()[self.depth] != 0)) {
-            serializer.serialize_seq(Some(0)).unwrap().end()
+            ZeroListSerializer::new().serialize(serializer)
         } else if !self.children.is_empty() {
             let mut seq = serializer.serialize_seq(None).unwrap();
             for child in &self.children {
@@ -395,7 +372,7 @@ impl<'a> NumpyF64Array<'a> {
 }
 
 impl<'a> Serialize for NumpyF64Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -414,6 +391,7 @@ pub struct DataTypeF64 {
 }
 
 impl Serialize for DataTypeF64 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -434,7 +412,7 @@ impl<'a> NumpyF32Array<'a> {
 }
 
 impl<'a> Serialize for NumpyF32Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -453,6 +431,7 @@ struct DataTypeF32 {
 }
 
 impl Serialize for DataTypeF32 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -473,7 +452,7 @@ impl<'a> NumpyU64Array<'a> {
 }
 
 impl<'a> Serialize for NumpyU64Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -492,6 +471,7 @@ pub struct DataTypeU64 {
 }
 
 impl Serialize for DataTypeU64 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -512,7 +492,7 @@ impl<'a> NumpyU32Array<'a> {
 }
 
 impl<'a> Serialize for NumpyU32Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -531,6 +511,7 @@ pub struct DataTypeU32 {
 }
 
 impl Serialize for DataTypeU32 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -551,7 +532,7 @@ impl<'a> NumpyU16Array<'a> {
 }
 
 impl<'a> Serialize for NumpyU16Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -570,6 +551,7 @@ pub struct DataTypeU16 {
 }
 
 impl Serialize for DataTypeU16 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -590,7 +572,7 @@ impl<'a> NumpyI64Array<'a> {
 }
 
 impl<'a> Serialize for NumpyI64Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -609,6 +591,7 @@ pub struct DataTypeI64 {
 }
 
 impl Serialize for DataTypeI64 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -629,7 +612,7 @@ impl<'a> NumpyI32Array<'a> {
 }
 
 impl<'a> Serialize for NumpyI32Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -648,6 +631,7 @@ pub struct DataTypeI32 {
 }
 
 impl Serialize for DataTypeI32 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -668,7 +652,7 @@ impl<'a> NumpyI16Array<'a> {
 }
 
 impl<'a> Serialize for NumpyI16Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -687,6 +671,7 @@ pub struct DataTypeI16 {
 }
 
 impl Serialize for DataTypeI16 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -707,7 +692,7 @@ impl<'a> NumpyI8Array<'a> {
 }
 
 impl<'a> Serialize for NumpyI8Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -726,6 +711,7 @@ pub struct DataTypeI8 {
 }
 
 impl Serialize for DataTypeI8 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -746,7 +732,7 @@ impl<'a> NumpyU8Array<'a> {
 }
 
 impl<'a> Serialize for NumpyU8Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -765,6 +751,7 @@ pub struct DataTypeU8 {
 }
 
 impl Serialize for DataTypeU8 {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -785,7 +772,7 @@ impl<'a> NumpyBoolArray<'a> {
 }
 
 impl<'a> Serialize for NumpyBoolArray<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -804,6 +791,7 @@ pub struct DataTypeBool {
 }
 
 impl Serialize for DataTypeBool {
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -879,7 +867,7 @@ pub struct NumpyInt8 {
 }
 
 impl Serialize for NumpyInt8 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -896,7 +884,7 @@ pub struct NumpyInt16 {
 }
 
 impl Serialize for NumpyInt16 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -913,7 +901,7 @@ pub struct NumpyInt32 {
 }
 
 impl Serialize for NumpyInt32 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -930,7 +918,7 @@ pub struct NumpyInt64 {
 }
 
 impl Serialize for NumpyInt64 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -947,7 +935,7 @@ pub struct NumpyUint8 {
 }
 
 impl Serialize for NumpyUint8 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -964,7 +952,7 @@ pub struct NumpyUint16 {
 }
 
 impl Serialize for NumpyUint16 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -981,7 +969,7 @@ pub struct NumpyUint32 {
 }
 
 impl Serialize for NumpyUint32 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -998,7 +986,7 @@ pub struct NumpyUint64 {
 }
 
 impl Serialize for NumpyUint64 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -1015,7 +1003,7 @@ pub struct NumpyFloat32 {
 }
 
 impl Serialize for NumpyFloat32 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -1032,7 +1020,7 @@ pub struct NumpyFloat64 {
 }
 
 impl Serialize for NumpyFloat64 {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -1049,7 +1037,7 @@ pub struct NumpyBool {
 }
 
 impl Serialize for NumpyBool {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -1083,7 +1071,6 @@ pub enum NumpyDatetimeUnit {
 
 impl fmt::Display for NumpyDatetimeUnit {
     #[cold]
-    #[cfg_attr(feature = "optimize", optimize(size))]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let unit = match self {
             Self::NaT => "NaT",
@@ -1114,7 +1101,6 @@ enum NumpyDateTimeError {
 
 impl NumpyDateTimeError {
     #[cold]
-    #[cfg_attr(feature = "optimize", optimize(size))]
     fn into_serde_err<T: ser::Error>(self) -> T {
         let err = match self {
             Self::UnsupportedUnit(unit) => format!("unsupported numpy.datetime64 unit: {}", unit),
@@ -1136,6 +1122,7 @@ impl NumpyDatetimeUnit {
     /// object rather than using the `descr` field of the `__array_struct__`
     /// because that field isn't populated for datetime64 arrays; see
     /// https://github.com/numpy/numpy/issues/5350.
+    #[cold]
     #[cfg_attr(feature = "optimize", optimize(size))]
     fn from_pyobject(ptr: *mut PyObject) -> Self {
         let dtype = ffi!(PyObject_GetAttr(ptr, DTYPE_STR));
@@ -1173,6 +1160,7 @@ impl NumpyDatetimeUnit {
     /// Return a `NumpyDatetime64Repr` for a value in array with this unit.
     ///
     /// Returns an `Err(NumpyDateTimeError)` if the value is invalid for this unit.
+    #[cold]
     #[cfg_attr(feature = "optimize", optimize(size))]
     fn datetime(&self, val: i64, opts: Opt) -> Result<NumpyDatetime64Repr, NumpyDateTimeError> {
         match self {
@@ -1245,7 +1233,7 @@ impl<'a> NumpyDatetime64Array<'a> {
 }
 
 impl<'a> Serialize for NumpyDatetime64Array<'a> {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
@@ -1313,7 +1301,7 @@ impl DateTimeLike for NumpyDatetime64Repr {
 }
 
 impl Serialize for NumpyDatetime64Repr {
-    #[cfg_attr(feature = "optimize", optimize(size))]
+    #[cold]
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
